const { consoleError, consoleSuccess } = require('./console');
const path = require('path');
const fs = require('fs');
const inquirer = require('inquirer');
const { copy } = require('./util');

const cwd = process.cwd();

const templateDir = path.resolve(__dirname, './template');

const renameFiles = {
  _gitignore: '.gitignore'
};

const extensionNamePromptConfig = {
  type: 'input',
  name: 'extensionName',
  message: 'Enter the extension name:',
  prefix: '*'
};

async function promptWhenAddExtension(before = []) {
  return inquirer.prompt([
    ...before,
    {
      type: 'input',
      name: 'store',
      message: 'Enter your store URL(example: https://xxx.myshoplaza.com/):',
      prefix: '*'
    },
    {
      type: 'input',
      name: 'token',
      message: 'Enter the store\'s token:',
      prefix: '*'
    },
    extensionNamePromptConfig
  ]);
}

/**
 * 使用cli创建一个项目
 * @param {} command 项目名
 * @returns
 */
async function createProject(command) {
  const { projectName, extensionName, store, token } = await promptWhenAddExtension([
    {
      type: 'input',
      name: 'projectName',
      message: 'Please input your project name:',
      prefix: '*'
    }
  ]);
  const projectDir = projectName.replace(/\/+$/g, '');
  const root = path.join(cwd, projectDir);
  if (fs.existsSync(root)) {
    consoleError(`the directory '${projectDir}' is exist.`);
    return;
  }
  if (!extensionName.trim()) {
    consoleError('extension id is required');
    return;
  }
  fs.mkdirSync(root, { recursive: true });

  const write = (file, content) => {
    const targetPath = path.join(root, renameFiles[file] ?? file);
    if (content) {
      fs.writeFileSync(targetPath, content);
    } else {
      copy(path.join(templateDir, file), targetPath);
    }
  };

  const files = fs.readdirSync(templateDir);
  for (const file of files.filter((f) => !['package.json'].includes(f))) {
    write(file);
  }

  let projectConfigContent = fs.readFileSync(path.join(templateDir, 'extension.config.js'), 'utf-8');
  projectConfigContent = projectConfigContent.replace('{store}', store).replace('{token}', token);
  write('extension.config.js', projectConfigContent);

  const pkg = JSON.parse(fs.readFileSync(path.join(templateDir, `package.json`), 'utf-8'));
  pkg.name = projectDir;
  write('package.json', JSON.stringify(pkg, null, 2) + '\n');

  fs.renameSync(path.join(root, 'extensions/extension-template'), path.join(root, `extensions/${extensionName}`));

  const extensionConfig = JSON.parse(
    fs.readFileSync(path.join(path.resolve(root, `extensions/${extensionName}`), `extension.json`), 'utf-8')
  );
  extensionConfig.extensionName = extensionName;
  write(`extensions/${extensionName}/extension.json`, JSON.stringify(extensionConfig, null, 2) + '\n');
  consoleSuccess(`Successfully created extension project '${projectName}'.`);
}

/**
 * 使用cli在现有项目中新增一个 extension
 */
async function createExtension(command) {
  const { extensionName } = await inquirer.prompt([extensionNamePromptConfig]);
  const extensionsDir = path.join(cwd, 'extensions');
  if (!fs.existsSync(extensionsDir)) {
    consoleError('Please check current dir is a extension project');
    return;
  }

  const targetDir = path.join(extensionsDir, extensionName);
  if (fs.existsSync(targetDir)) {
    consoleError(`This '${extensionName}' extension already exists.`);
    return;
  }
  fs.mkdirSync(targetDir);

  const extensionTemplateDir = path.join(templateDir, 'extensions', 'extension-template');
  const files = fs.readdirSync(extensionTemplateDir);
  for (const file of files.filter((f) => f !== 'extension.json')) {
    copy(path.join(extensionTemplateDir, file), path.join(targetDir, file));
  }

  const extensionConfig = JSON.parse(fs.readFileSync(path.join(extensionTemplateDir, `extension.json`), 'utf-8'));
  extensionConfig.extensionName = extensionName;
  fs.writeFileSync(path.join(targetDir, 'extension.json'), JSON.stringify(extensionConfig, null, 2) + '\n');
  consoleSuccess(`Successfully created extension '${extensionName}'.`);
}

module.exports = {
  createProject,
  createExtension
};
