const inquirer = require('inquirer');
const {
  getExtensionPathById,
  getExtensionInfo,
  copy,
  getDistPath,
  getExtensionsPath,
  getExtensionConfig
} = require('../util');
const { cwd } = require('../config');
const path = require('path');
const chokidar = require('chokidar');
const fs = require('fs');
const { viteBuild } = require('../build');
const { consoleError, consoleBlue, consoleSuccess } = require('../console');

async function buildExtension(id) {
  return viteBuild({ id })
    .catch(consoleError)
    .then(() => {
      const { distName } = getExtensionInfo(id);
      return distName;
    });
}

let webSocket = undefined;

function handleWsConnection(ws) {
  webSocket = ws;
  ws.on('error', console.error);
  ws.on('message', function message(data) {
    console.log(`Received message ${data} from user `);
  });

  const extensionList = [];
  fs.readdirSync(getExtensionsPath()).forEach((ext) => {
    extensionList.push(generateExtensionData(ext));
  });
  ws.send(JSON.stringify({ event: 'init', data: extensionList }));

}

async function createWsServer(httpServer) {
  const { WebSocketServer } = require('ws');
  const wss = new WebSocketServer({ noServer: true });
  wss.on('connection', handleWsConnection);

  httpServer.on('upgrade', function upgrade(request, socket, head) {
    wss.handleUpgrade(request, socket, head, function done(ws) {
      wss.emit('connection', ws, request);
    });
  });
  return wss;
}

function createServer(dir) {
  const connect = require('connect');
  const http = require('http');
  const app = connect();

  const serveStatic = require('serve-static');
  app.use(
    serveStatic(dir, {
      setHeaders: function (res) {
        res.setHeader('Access-Control-Allow-Origin', '*');
        res.setHeader('Access-Control-Allow-Methods', 'GET');
        res.setHeader('Access-Control-Allow-Headers', 'Content-Type');
      }
    })
  );
  const httpServer = http.createServer(app);

  return httpServer;
}

function generateExtensionData(id) {
  const info = getExtensionInfo(id);
  const config = getExtensionConfig(id);
  const data = {
    extensionId: info.id,
    resourceUrl: `http://localhost:8888/${info.distName}`,
    status: 'draft',
    version: config.version,
    fields: JSON.stringify(config)
  };
  return data;
}

async function startDev() {
  if (fs.existsSync(getDistPath())) {
    fs.rmdirSync(getDistPath(), { recursive: true });
  }

  const extensionPath = path.resolve(cwd, './extensions');
  const files = fs.readdirSync(extensionPath);
  const choices = files.filter((file) => {
    return fs.statSync(path.resolve(extensionPath, file)).isDirectory();
  });
  const { extensionNameList } = await inquirer.prompt([
    {
      type: 'checkbox',
      name: 'extensionNameList',
      message: 'Please select one or more extensions to develop:',
      prefix: '*',
      loop: false,
      choices,
      validate(answer) {
        if (answer.length === 0) {
          return 'You must choose at least one extension.';
        }
        return true;
      },
    }
  ]);

  // build
  const willBuildExts = extensionNameList;
  await Promise.all(willBuildExts.map((id) => buildExtension(id)));

  // 复制index.js文件
  copy(path.join(__dirname, 'client.js'), path.join(getDistPath(), 'index.js'));

  // 监听文件改变
  chokidar.watch(getExtensionsPath()).on('change', async (event) => {
    const id = event.split('extensions/').pop().split('/').shift();
    if (id) {
      consoleBlue(`update extension: ${id}`);
      await buildExtension(id);
      webSocket && webSocket.send(JSON.stringify({ event: 'update', data: generateExtensionData(id) }));
    }
  });

  // 启动服务
  const httpServer = createServer(getDistPath());
  createWsServer(httpServer, willBuildExts);
  httpServer.listen(8888);
  consoleSuccess('started server on url: http://localhost:8888/index.js');
}

module.exports = {
  startDev
};
